/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.da;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCDialog;
import net.sf.dc.console.components.FileField;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.Repository;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.export.TableToFileExporter;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.BrowserLauncher;
import net.sf.ext.util.FileFilter;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Via this dialog CD content can be imported. The import type is
 * determined by the specified module. The imported data is transfered
 * to the insert table. Here, the user can edit the values and commits
 * the data to the database 
 * 
 * @author Robert Jan van der Waals
 * @since 1.4
 * @version 1.5
 */
public class ExportToPdfDialog extends DCDialog {

    private TableToFileExporter process;
    
    private final Color color = 
        Settings.getValueAsColor(Repository.settings.stColor);
    
    private final JButton buttonStartExport = 
        ComponentFactory.getButton(Resources.getText(DCLabels.lblStart));
    private final JButton buttonCancel = 
        ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));
    private final JButton buttonClose = 
        ComponentFactory.getButton(Resources.getText(DCLabels.lblClose));
    private final JButton buttonResults = 
        ComponentFactory.getButton(Resources.getText(DCLabels.lblOpenReport));
    private final FileField fileField;
    
    private final FileFilter pdfFilter = new FileFilter("pdf");
    private final FileFilter htmlFilter = new FileFilter("html");
    
    private final JTextArea textLog = ComponentFactory.getTextArea();
    private final JProgressBar progressBar = new JProgressBar();
    
    private final DCObject[] objects;

    public ExportToPdfDialog(DCObject[] objects) {
        super(DataCrow.mainFrame);
        
        this.objects = objects;
        
        setHelpIndex("dc.Tools.Reporting");
        
        fileField = ComponentFactory.getFileField(true, false, pdfFilter);
        buildDialog();
    }
    
    /**
     * Initializes the progress bar. Sets the maximum value
     * @param maxValue maximum value
     */
    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }
    
    /**
     * Updates the value of the progress bar
     * @param value progress value
     */
    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }  
    
    /**
     * Adds a log message
     * @param message message to be added
     */
    public void addMessage(String message) {
        textLog.insert(message + '\n', 0);
    }
    
    public void allowActions(boolean b) {
        buttonStartExport.setEnabled(b);
        buttonResults.setEnabled(b);
    }
    
    /**
     * Opens the PDF file in an external program
     * @see BrowserLauncher
     */
    private void showResults() {
    	File file = fileField.getFile();
        if (file != null && file.exists()) {
            try {
            	BrowserLauncher.openURL(file.toString());
            } catch (Exception exp) {
                new MessageBox(Resources.getText(DCMessages.msgErrorWhileOpeningX,
                               new String[] {file.toString(), exp.getMessage()}), MessageBox._WARNING);
                Messages.setMessage(exp);
            }
        } else {
        	new MessageBox(Resources.getText(DCMessages.msgCouldNotOpenReport), MessageBox._WARNING);
        }
    }
    
    private void startExport() {
    	File file = fileField.getFile();
        if (file != null) {
        	process = new TableToFileExporter(this, objects, TableToFileExporter._PDF, file);
            allowActions(false);
            process.start();
        } else {
            new MessageBox(Resources.getText(DCMessages.msgSelectTargetFile), MessageBox._WARNING);
        }
    }
    
    private void cancel() {
        if (process != null) {
            process.cancel();
        }
        allowActions(true);
    }
    
    private void close() {
        cancel();
        dispose();
        setVisible(false);
    }
    
    private void buildDialog() {
        //**********************************************************
        //Actions Panel
        //**********************************************************
        JPanel panelInput = new JPanel(false);
        panelInput.setLayout(Layout.getGBL());
        panelInput.setBackground(color);
        
        panelInput.add(fileField,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                      ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                       new Insets( 5, 5, 5, 5), 0, 0));
        panelInput.setBorder(ComponentFactory.getTitleBorder(Resources.getText(DCLabels.lblTargetFile)));
        
    	//**********************************************************
        //Actions Panel
        //**********************************************************
        JPanel panelActions = new JPanel(false);
        panelActions.setLayout(Layout.getGBL());
        panelActions.setBackground(color);

        buttonStartExport.addActionListener(new StartExportButtonAction());
        buttonCancel.addActionListener(new CancelButtonAction());
        buttonResults.addActionListener(new ResultsButtonAction());
        buttonClose.addActionListener(new CloseButtonAction());
        
        buttonStartExport.setMnemonic(KeyEvent.VK_S);
        buttonCancel.setMnemonic(KeyEvent.VK_X);
        buttonClose.setMnemonic(KeyEvent.VK_C);
        buttonResults.setMnemonic(KeyEvent.VK_R);
        
        panelActions.add(buttonStartExport,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel,    Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonResults,   Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonClose,     Layout.getGBC( 3, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        
        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel(false);
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.setBackground(color);
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0)); 
        
        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel(false);
        panelLog.setLayout(Layout.getGBL());
        panelLog.setBackground(color);
        
        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));
        
        panelLog.setBorder(ComponentFactory.getTitleBorder(
                           Resources.getText(DCLabels.lblLog)));        
        
                    
        //**********************************************************
        //Main Panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(
                panelInput,    Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
               ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                new Insets( 0, 0, 0, 0), 0, 0));                 
        this.getContentPane().add(
                 panelActions, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));                 
        this.getContentPane().add(
                 panelProgress, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 0), 0, 0));                 
        this.getContentPane().add(
                 panelLog,      Layout.getGBC( 0, 3, 1, 1, 40.0, 40.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));                 

        getContentPane().setBackground(color);
        
        this.pack();
        setSize(Settings.getValueAsDimension(Repository.settings.stImportCDDialogSize));
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
        
        this.setResizable(true);
    }
    
    private class ResultsButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showResults();
        }
    }
    
    private class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }    
    
    private class CancelButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancel();
        }
    }
    
    private class StartExportButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startExport();
        }
    }    
}
