/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.console.da;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import net.sf.dc.console.MainFrame;
import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.console.components.DCFrame;
import net.sf.dc.console.components.FileField;
import net.sf.dc.console.components.Layout;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.DataCrow;
import net.sf.dc.core.IconLibrary;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.processes.cdimport.ImportCDProcess;
import net.sf.dc.resources.DCLabels;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Via this dialog CD content can be imported. The import type is
 * determined by the specified module. The imported data is transfered
 * to the insert table. Here, the user can edit the values and commits
 * the data to the database 
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.0
 */
public class ImportCDDialog extends DCFrame {
    private final Color color = 
        Settings.getValueAsColor(Repository.settings.stColor);
    private JCheckBox checkVerify;    
    private FileField fieldLocation;
    private JTextField textCDName;

    private final JButton buttonCDExport = 
        ComponentFactory.getButton(Resources.getText(DCLabels.lblStartImport));
    private final JButton buttonCancel = 
        ComponentFactory.getButton(Resources.getText(DCLabels.lblCancel));
    private final JButton buttonClose = 
        ComponentFactory.getButton(Resources.getText(DCLabels.lblClose));
    private final JButton buttonResults = 
        ComponentFactory.getButton(Resources.getText(DCLabels.lblResults));
    
    private JTextArea textLog = ComponentFactory.getTextArea();
    private JProgressBar progressBar = new JProgressBar();
    
    private ImportCDProcess process;
    
    private DCModule module;

    public ImportCDDialog(String title) {
        super();
        
        setHelpIndex("dc.Tools.DataImport");
        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
            	close();
            }
        });
        
        Image image = IconLibrary._icoImport.getImage();
        setIconImage(image);
        image.flush();  
        setTitle(title);  
        
        buildDialog();
        
        Dimension dim = Settings.getValueAsDimension(Repository.settings.stImportCDDialogSize);
        setSize(dim);
    }
    
    /**
     * Initializes the progress bar. Sets the maximum value
     * @param maxValue maximum value
     */
    public void initProgressBar(int maxValue) {
        progressBar.setValue(0);
        progressBar.setMaximum(maxValue);
    }
    
    /**
     * Updates the value of the progress bar
     * @param value progress value
     */
    public void updateProgressBar(int value) {
        progressBar.setValue(value);
    }  
    
    /**
     * Adds a log message
     * @param message message to be added
     */
    public void addMessage(String message) {
        textLog.insert(message + '\n', 0);
    }      

    /**
     * Start the import of the CD Content. The specified module is 
     * used to get the actual import job. The import is run in a 
     * seperate thread
     */
    protected void startImport() {
        boolean bVerify = checkVerify.isSelected();
        
        String sourceDir = "";
        if (fieldLocation.getFile() != null) {
            sourceDir = fieldLocation.getFileAsString();
        }
        
        if (sourceDir == null || sourceDir.equals("")) {
            new MessageBox(this, Resources.getText(DCMessages.msgSelectDirectory), 
                           MessageBox._WARNING);
            return;
        }
        
        String cdName = textCDName.getText().trim();
        if (cdName.equals("")) {
            new MessageBox(this, Resources.getText(DCMessages.msgEnterCDName), 
                           MessageBox._WARNING);
            return;
        }
        
        denyActions();
        
        process = module.getCDImportProcess(this, sourceDir, bVerify, cdName);
        process.setPriority(5);
        process.start();
    }   
    
    /**
     * Disables the action buttons
     */
    public void denyActions() {
        progressBar.setValue(0);
        textLog.setText("");

    	buttonCDExport.setEnabled(false);
        buttonResults.setEnabled(false);
    }

    /**
     * Enables the action buttons
     */
    public void allowActions() {
        buttonCDExport.setEnabled(true);
        buttonResults.setEnabled(true);
    }
    
    /**
     * Stops the import and closes the form
     */
    private void close() {
    	cancel();
        dispose();
        Settings.setValue(Repository.settings.stImportCDDialogSize, getSize());
        setVisible(false);
    }

    /**
     * Stops the import
     */
    private void cancel() {
        allowActions();
    	if (process != null) {
    		process.cancel();
        }
    }
    
    private void showResults() {
    	DataCrow.mainFrame.setSelectedTab(MainFrame._INSERTTAB);
    }
    
    /**
     * Sets the module that is to be used to get the actual import job.
     * @param module the module from which to get the actual import
     */
    public void setModule(DCModule module) {
        this.module = module;
    }
    
    private void buildDialog() {
        //**********************************************************
        //Input panel
        //**********************************************************
        JPanel panelInput = new JPanel(false);
        panelInput.setLayout(Layout.getGBL());
        panelInput.setBackground(color);
        panelInput.setBorder(ComponentFactory.getTitleBorder(""));

        JLabel labelLocation = ComponentFactory.getLabel(Resources.getText(DCLabels.lblCDLocation));
        fieldLocation = new FileField(false, true);
        checkVerify = ComponentFactory.getCheckBox(Resources.getText(DCLabels.lblAskPerDir));
        JLabel labelCDName = ComponentFactory.getLabel(Resources.getText(DCLabels.lblLocationName));
        textCDName = ComponentFactory.getTextField();

        
        checkVerify.setBackground(color);

        panelInput.add(  labelLocation,  Layout.getGBC( 0, 0, 2, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  fieldLocation,  Layout.getGBC( 1, 0, 3, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  labelCDName,    Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  textCDName,     Layout.getGBC( 1, 1, 2, 1, 20.0, 20.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(0, 5, 0, 5), 0, 0));
        panelInput.add(  checkVerify,    Layout.getGBC( 0, 2, 3, 1, 1.0, 1.0
                                        ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL,
                                         new Insets(5, 5, 0, 5), 0, 0));
                                            
        //**********************************************************
        //Actions Panel
        //**********************************************************
        JPanel panelActions = new JPanel(false);
        panelActions.setLayout(Layout.getGBL());
        panelActions.setBackground(color);

        buttonCDExport.addActionListener(new ExportCDButtonAction());
        buttonCancel.addActionListener(new CancelButtonAction());
        buttonResults.addActionListener(new ResultsButtonAction());
        buttonClose.addActionListener(new CloseButtonAction());
        
        buttonCDExport.setMnemonic(KeyEvent.VK_S);
        buttonCancel.setMnemonic(KeyEvent.VK_X);
        buttonResults.setMnemonic(KeyEvent.VK_R);
        buttonClose.setMnemonic(KeyEvent.VK_C);
        
        panelActions.add(buttonCDExport,  Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonCancel,    Layout.getGBC( 1, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
										  new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonResults,   Layout.getGBC( 2, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        panelActions.add(buttonClose,     Layout.getGBC( 3, 0, 1, 1, 1.0, 1.0
                                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                                          new Insets( 5, 5, 5, 5), 0, 0));
        
        //**********************************************************
        //Progress panel
        //**********************************************************
        JPanel panelProgress = new JPanel(false);
        panelProgress.setLayout(Layout.getGBL());
        panelProgress.setBackground(color);
        panelProgress.add(progressBar, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                         ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                          new Insets(5, 5, 5, 5), 0, 0)); 
        
        //**********************************************************
        //Log Panel
        //**********************************************************
        JPanel panelLog = new JPanel(false);
        panelLog.setLayout(Layout.getGBL());
        panelLog.setBackground(color);
        
        JScrollPane scroller = new JScrollPane(textLog);
        scroller.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        panelLog.add(scroller, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                    ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                     new Insets(5, 5, 5, 5), 0, 0));
        
        panelLog.setBorder(ComponentFactory.getTitleBorder(
                           Resources.getText(DCLabels.lblLog)));        
        
                    
        //**********************************************************
        //Main Panel
        //**********************************************************
        this.getContentPane().setLayout(Layout.getGBL());
        this.getContentPane().add(
                 panelInput,      Layout.getGBC( 0, 0, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 5, 5, 5, 5), 0, 0));
        this.getContentPane().add(
                 panelActions, Layout.getGBC( 0, 1, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                 new Insets( 0, 0, 0, 0), 0, 0));                 
        this.getContentPane().add(
                 panelProgress, Layout.getGBC( 0, 2, 1, 1, 1.0, 1.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.HORIZONTAL,
                 new Insets( 0, 0, 0, 0), 0, 0));                 
        this.getContentPane().add(
                 panelLog, Layout.getGBC( 0, 3, 1, 1, 40.0, 40.0
                ,GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
                 new Insets( 0, 0, 0, 0), 0, 0));                 

        getContentPane().setBackground(color);
        
        this.pack();
        setSize(Settings.getValueAsDimension(Repository.settings.stImportCDDialogSize));
        
        Dimension location = Utilities.getCenteredWindowLocation(getSize());
        setLocation(location.width, location.height);
        
        this.setResizable(true);
    }

    private class ResultsButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            showResults();
        }
    }
    
    private class CloseButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            close();
        }
    }    
    
    private class CancelButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            cancel();
        }
    }
    
    private class ExportCDButtonAction implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            startImport();
        }
    }
}
