/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.core.objects;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.swing.ImageIcon;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.db.QueryCondition;
import net.sf.dc.db.QueryOptions;
import net.sf.dc.exceptions.ValidationException;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.wf.WorkFlow;
import net.sf.dc.wf.requests.IRequest;
import net.sf.dc.wf.requests.RequestCollection;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.Resources;
import net.sf.ext.util.Utilities;

/**
 * Base class for all Data Crow objects. Here the fields, values and required
 * fields are stored. Checks are done to garantee uniqueness and validity
 * of the items.
 *
 * Objects which extend this class need to override the save and load methods
 *
 * @author Robert Jan van der Waals
 * @since 1.3
 * @version 0.50
 */
public abstract class DCObject {

    protected int uiRowNumber = -1;
    protected int module;
    protected ArrayList alValues = new ArrayList();
    
    /** Indicates of the data should be checked for validity */
    protected boolean bValidation = true;
    /** Indicates if queries should display output */
    private boolean bSilence = false;
    private RequestCollection requests = new RequestCollection();    
    protected static boolean alwaysGenerateID = false;
    protected static final WorkFlow wf = new WorkFlow();
    
    protected String sTableName = "";
    protected String sSelectTableDef = "";
    
    protected ArrayList alChildren = new ArrayList();
    
    private boolean isBatch = false; 
    private boolean endOfBatch = false;
    
    public static final int _ID = 0;
    
    /**
     * Creates an empty data crow object
     */
    protected DCObject() {
        setModuleIndex();
        setTableName();
        setSelectTableDef();
        initializeSettings();
        initializeValues();
    }

    /**
     * Remove all requestors listening in on this object
     */
    public void removeRequestors() {
        requests.clear();
    }
    
    /**
     * Returns all requestors
     */
    public RequestCollection getRequests() {
    	return requests;
    }

    /**
     * Add a requestor. The requestor will be executed once the object
     * is saved / deleted or updated
     * @param requestor
     */
    public void addRequest(IRequest request) {
        requests.addRequest(request);
    }
    
    /**
     * Indicates this object to be the last in row of an update / insert or delete
     * @param end
     */
    public void setEndOfBatch(boolean end) {
    	endOfBatch = end;
    }
    
    /**
     * Indicates this object being part of a larger operation
     * @param batch part of batch y/n
     */
    public void setBatch(boolean batch) {
    	isBatch = batch;
    }
    
    /**
     * Part of a batch ?
     */
    public boolean isBatch() {
        return isBatch;
    }

    /**
     * End of the batch ?
     */
    public boolean isEndOfBatch() {
    	return endOfBatch;
    }
    
    /**
     * Remove all children from this object
     */
    public void removeChildren() {
    	alChildren.clear();
        alChildren = new ArrayList();
    }
    
    public void addChildren(Collection children) {
        alChildren.addAll(children);
    }
    
    /**
     * Adds a child to this object
     * @param child
     */
    public void addChild(DCObject child) {
        child.setBatch(isBatch());
        
        if (endOfBatch) {
        	setEndOfBatch(false);
            
            for (Iterator iter = alChildren.iterator(); iter.hasNext(); ) {
                ((DCObject) iter.next()).setEndOfBatch(false);    
            }
            child.setEndOfBatch(true);
        }
        
        if (!alChildren.contains(child)) {
            alChildren.add(child);
        }
    }
    
    /**
     * Retrieve all children of this object
     */
    public Collection getChildren() {
        return alChildren;
    }

    /**
     * Retrieve the field index holding the reference to the parent 
     * (the primary key of the parent)
     */
    public int getParentReferenceFieldIndex() {
    	DCFieldDefinition[] fields = getFields();
        for (int i = 0; i < fields.length; i++) {
        	if (fields[i].getValueType() == Repository.datatypes._PARENTREFERENCE) {
        		return i;
            }
        }
        return -1;
    }
    
    /**
     * Sets a complete set of values on the object
     * calls setValue for each individual value 
     * @see #setValue(int, Object)
     */
    public void setValues(Object[] row) {
        int counter = 0;
        if (row.length == DCModules.getModule(module).getFieldCount()) {
            // ID is included in the array
            counter++;
            if (!alwaysGenerateID) {
                setValue(_ID, row[0]);
            }
        }

        for (int i = counter; i < row.length; i++) {
            setValue(i, row[i]);            
        }
    }      
    
    /**
     * Returns the icon for this object. Example; Software Categories
     * has an icon  
     */
    public abstract ImageIcon getIcon();
    
    /**
     * Returns the module to which this object belongs.
     * Shortcut for getting the module object instead of the 
     * module index.
     */
    public DCModule getModule() {
        return DCModules.getModule(module);
    }
    
    /**
     * Returns all the field definitions. 
     * Shortcut; normally you would first have to ask for the
     * module and then ask to the module which field definitions
     * it contains. 
     */
    public DCFieldDefinition[] getFields() {
        return getModule().getFieldDefinitions();
    }
    
    /**
     * Sets all fields to 'unchanged'. This garantees that none of the
     * values will be saved to the database when performing an update
     * or insert. All 'setValue' calls are up to this moment are 
     * discarded
     */
    public void undoChanges() {
        for (Iterator iter = alValues.iterator(); iter.hasNext(); ) {
            DCValue value = (DCValue) iter.next();
            value.setChanged(false);
        }
    }
    
    /**
     * Indicates if changes have been made to this data crow object.
     * A field is indicated as beign changed when the user or system
     * has performed a set value.
     */
    public boolean isChanged() {
        for (Iterator iter = alValues.iterator(); iter.hasNext(); ) {
            DCValue value = (DCValue) iter.next();
            if (value.isChanged()) {
                return true;
            }
        }        
        return false;
    }

    /**
     * Indicates if changes have been made to the specified value.
     * A field is indicated as beign changed when the user or system
     * has performed a set value.
     * 
     * @param index the value index to check on
     */
    public boolean isChanged(int index) {
        return getValueDef(index).isChanged();
    }
    
    /**
     * Marks the specified value as changed (or not) This means that 
     * the value will be considered for an insert or update to the 
     * database
     * 
     * @param index the value index to mark as changed / unchanged
     * @param b changed y/n 
     */    
    public void setChanged(int index, boolean b) {
        Object o = getValue(index);
        if (!o.equals(new Integer(-1))) {
        	getValueDef(index).setChanged(b);
        }
    }    

    /**
     * Indicates if the given field is visible to the user or not
     * @param index field index
     */
    public boolean isHidden(int index) {
        return getFields()[index].isHidden();
    }
    
    /**
     * Indicates whether the given field is required or not
     * @param index field index
     */
    public boolean isRequired(int index) {
        return getFields()[index].isRequired();
    }

    /**
     * Indicates whether the given field is required or not
     * @param index field index
     */
    public boolean isSearchable(int index) {
        return getFields()[index].isSearchable();
    }    
    
    /**
     * Retrieves all currently, to the user, hidden fields
     */
    public int[] getHiddenFields() {
        DCFieldDefinition[] fields = getFields();
        
        int counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (fields[i].isHidden()) {
                counter++;
            }
        }
        int[] hidden = new int[counter];
        counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (fields[i].isHidden()) {
                hidden[counter++] = i;
            }
        }
        return hidden;
    }    

    /**
     * Retrieves the ID for this data crow object
     */
    public String getID() {
        return getValueDef(_ID).getValueAsString();
    }
    
    /**
     * Retrieves all fields on which a search cannot be performed.
     * Example; a user can not perform a search on a picture field
     */
    public int[] getNotSearchableFields() {
        DCFieldDefinition[] fields = getFields();
        
        int counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (!fields[i].isSearchable()) {
                counter++;
            }
        }
        int[] notSearchable = new int[counter];
        counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (!fields[i].isSearchable()) {
                notSearchable[counter++] = i;
            }
        }
        return notSearchable;    
    }
    
    /**
     * Retrieves the indices of the, for the user visible fields
     */
    public int[] getVisibleFields() {
        DCFieldDefinition[] fields = getFields();
        
        int counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (!fields[i].isHidden()) {
                counter++;
            }
        }
        int[] visible = new int[counter];
        counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (!fields[i].isHidden()) {
                visible[counter++] = i;
            }
        }
        return visible;
    }

    /**
     * Retrieves the values for the visible fields
     */
    public Object[] getVisibleValues() {
        int[] visible = getVisibleFields();
        
        Object[] o = new Object[visible.length];
        for (int i = 0; i < visible.length; i++) {
            o[i] = getValueDef(visible[i]).getValue();
        }
        return o;
    }

    /**
     * Hack: always generate a new ID. Only used for upgrades of old
     * CD Library database. Otherwise: don't touch
     * 
     * @deprecated
     * @param alwaysGenerateID
     */
    public static void setAlwaysGenerateID(boolean alwaysGenerateID) {
        DCObject.alwaysGenerateID = alwaysGenerateID;
    }

    /**
     * Sets a value for the given field
     * @param iField field indicator
     * @param sValue value as String
     */
    public void setValue(int index, Object value) {
        getValueDef(index).setValue(value, 
                                    getFields()[index]);
    }

    /**
     * Sets all values to empty
     */
    public void clearValues() {
        int counter = 0;
        for (Iterator iter = alValues.iterator(); iter.hasNext(); ) {
            DCValue valueDef = (DCValue) iter.next();
            valueDef.setValue(null, getFields()[counter]);
        }
    } 

    /**
     * Retrieves all field labels 
     */
    public String[] getLabels() {
        DCFieldDefinition[] fields = getFields();
        String[] labels = new String[fields.length];
        for (int i = 0; i < fields.length; i++) {
            labels[i] = fields[i].getLabel();
        }
        return labels;
    }
    
    public String getSelectTableDef() {
        return sSelectTableDef;
    }
    
    /**
     * Returns the table name for this data crow object
     */
    public String getTableName() {
        return sTableName;
    }

    /**
     * Retrieves the label for the given field 
     * @param index field index
     */
    public String getLabel(int index) {
        return getFields()[index].getLabel();
    }

    /**
     * Retrieves the maximum length(s) for all the fields
     */
    public int[] getMaxFieldLengths() {
        DCFieldDefinition[] fields = getFields();
        int[] maxLength = new int[fields.length];
        for (int i = 0; i < fields.length; i++) {
            maxLength[i] = fields[i].getMaximumLength();
        }
        return maxLength;
    }

    /**
     * Retrieves the maximum length for the given field 
     * @param index field index
     */
    public int getMaxFieldLength(int index) {
        return getFields()[index].getMaximumLength();
    }
    
    /**
     * Returns the value of a field
     *
     * @param iField    field indicator
     * @return (String) field value
     */
    public Object getValue(int index) {
        return getValueDef(index).getValue();
    } 

    public String getDisplayString(int index) {
        if (getFieldType(index) != ComponentFactory._PICTUREFIELD) {
        	return getValueDef(index).getDisplayString(getFieldType(index));
        } else {
        	return "";
        }
    }
    
    public Object[] getValues() {
        Object[] values = new Object[alValues.size()];
        int counter = 0;
        for (Iterator iter = alValues.iterator(); iter.hasNext(); counter++){
        	values[counter] = ((DCValue) iter.next()).getValue();
        }
        return values;
    }

    public void setRow(int row) {
        uiRowNumber = row;
    }
    
    public int getRow() {
    	return uiRowNumber;
    }
    
    /**
     * Returns all the UI field types
     * @see net.sf.dc.console.components.ComponentFactory
     *
     * @return UI field type indices
     */
    public int[] getFieldTypes() {
        DCFieldDefinition[] fields = getFields();

        int[] fieldTypes = new int[fields.length];
        for (int i = 0; i < fields.length; i++) {
            fieldTypes[i] = fields[i].getFieldType();
        }
        return fieldTypes;
    }

    /**
     * Returns the UI field type for the given field
     * @see net.sf.dc.console.components.ComponentFactory
     *
     * @return UI field type index
     */    
    public int getFieldType(int index) {
        return getFields()[index].getFieldType();
    }

    /**
     * Returns all the database field names
     *
     * @return database field names array
     */
    public String[] getDatabaseFieldNames() {
        DCFieldDefinition[] fields = getFields();

        String[] fieldNames = new String[fields.length];
        for (int i = 0; i < fields.length; i++) {
            fieldNames[i] = fields[i].getDatabaseFieldName();
        }
        return fieldNames;
    }

    /**
     * Returns the database field name for the given field
     *
     * @param index field index
     */    
    public String getDatabaseFieldName(int index) {
        return getFields()[index].getDatabaseFieldName();
    }

    /**
     * Returns the required field names
     */
    public int[] getRequiredFields() {
        DCFieldDefinition[] fields = getFields();

        int counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (fields[i].isRequired()) {
                counter++;
            }
        }
        int[] required = new int[counter];
        counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (fields[i].isRequired()) {
                required[counter++] = i;
            }
        }
        return required; 
    }

	/**
	 * Returns the fields that are uneditable to the user
	 */
	public int[] getReadOnlyFields() {
        DCFieldDefinition[] fields = getFields();

        int counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (fields[i].isReadOnly()) {
                counter++;
            }
        }
        int[] readonly = new int[counter];
        counter = 0;
        for (int i = 0; i < fields.length; i++) {
            if (fields[i].isReadOnly()) {
                readonly[counter++] = i;
            }
        }
        return readonly; 
	}

    /**
     * Inserts a new item. Requests to the UI can be passed.
     */
    public void saveNew() throws ValidationException {
        //setIDs();
        checkIntegrity(false);
        wf.insertValues(this);
    }

    /**
     * Updates an item. Requests to the UI can be passed.
     */
    public void saveUpdate() throws ValidationException {
        this.checkIntegrity(true);
        wf.updateValues(this);
    }
    
    public void delete() {
        wf.deleteValues(this);
    }

    /**
     * Loads items. Requests to the UI can be passed.
     * @param options query options, can be null
     * @param requestors  collection of requests (can be null)
     */
    public void load(QueryOptions options) {
        wf.searchValues(this, options);
    }

    /**
     * Disables the output of this query to the log and log screen.
     * Also disables showing messageboxes when a query was success-
     * fully executed
     *
     * @param silence show output y/n
     */
    public void setSilence(boolean silence) {
        this.bSilence = silence;
    }

    /**
     * Indicates if queries should display output
     */
    public boolean getSilence() {
        return this.bSilence;
    }

    /**
     * Specifies if values should be validated
     * @param validate validate values y/n
     */
    public void setValidation(boolean validate) {
        bValidation = validate;
    }

    /**
     * Creates a query condition for select statements by using the
     * field names and their values
     * @param select indicates if this is about a select query
     */
    public QueryCondition toQueryCondition(boolean select) {
        QueryCondition condition = new QueryCondition(this, select);
        return condition;
    }
    
    /**
     * Returns the field definition object for the given field index
     * @param index field index
     */
    public DCFieldDefinition getField(int index) {
        return getFields()[index];
    }
    
    /**
     * Returns the labels of the fields that have been marked as changed
     */
    public String[] getChangedFields()  {
        ArrayList changedFields = new ArrayList();
        int counter = 0;
        for (Iterator iter = alValues.iterator(); iter.hasNext(); counter++) {
            DCValue value = (DCValue) iter.next();
            if (value.isChanged()) {
                changedFields.add(getField(counter).getDatabaseFieldName());
            }
        }
        String[] fieldNames = new String[changedFields.size()];
        return (String[]) changedFields.toArray(fieldNames);
    }
    
    /**
     * Returns the values of the fields that have been marked as changed
     */
    public Object[] getChangedValues() {
        ArrayList changedValues = new ArrayList();
        int counter = 0;
        for (Iterator iter = alValues.iterator(); iter.hasNext(); counter++) {
            DCValue val = (DCValue) iter.next();
            if (val.isChanged()) {
                changedValues.add(val.getValue());   
            }    
        }
        return changedValues.toArray();        
    }

    /**
     * Checks if the values are unique and if all the required fields
     * have been filled
     * @param update indicates if the values are new or existing
     */
    public void checkIntegrity(boolean update) throws ValidationException {
        if (Settings.getValueAsBoolean(Repository.settings.stCheckRequiredFields)) {
            validateRequiredFields();
        }
        if (Settings.getValueAsBoolean(Repository.settings.stCheckUniqueness)) {
            isUnique(this, update);
        }
    }

    /**
     * Returns the create statement for the table for this object 
     */
    public String[] getCreateStatements() {
        DCFieldDefinition[] fields = getFields();
        
        String[] sCreate = new String[fields.length];
        for (int i = 0; i < fields.length; i++) {
            sCreate[i] = fields[i].getDatabaseFieldName() + " " +
                         fields[i].getDataBaseFieldType();
            
            if (i == 0) {
                sCreate[i] += " PRIMARY KEY";
            }
        }
        return sCreate;
    }


    /**
     * Checks to see whether the item already exists or not
     *
     * @param o object to be checked
     * @param bUpdateQuery indicates whether the object is to 
     *        be updated or if it is a new object
     */
    public void isUnique(DCObject o, boolean bUpdateQuery) throws ValidationException {
        boolean bUnique = wf.checkUniqueness(o, bUpdateQuery);
        if (!bUnique && bValidation) {
            String message;
            if (getRow() != -1) {
                // shown by the table panels (ui row ID is set)
                message = Resources.getText(DCMessages.msgItemNotUniqueRow,
                          new String[] {toString(), String.valueOf(uiRowNumber + 1)});
            } else {
                // shown by item forms and the likes
                message = Resources.getText(DCMessages.msgItemNotUnique);
            }            
            
        	throw new ValidationException(message);
                    
        }
    }

    /**
     * Checks to see whether al required fields have been filled
     */
    public String getUnfilledRequiredFields() {
        int[] required = getRequiredFields();
        String s = ""; 
        for (int i = 0; i < required.length; i++) {
            Object value = getValueDef(required[i]).getValue();
            if (value == null || value.toString().equals("")) {
                if (i == required.length - 1) {
                    s += getLabel(required[i]);                    
                } else {
                    s += getLabel(required[i]) + ", ";
                }
            }
        }
        return s;
    }

    /**
     * Returns all field indices
     */
    public int[] getFieldIndices() {
        int[] indices = new int[getModule().getFieldCount()];
        for (int i = 0; i < indices.length; i++) {
            indices[i] = i;   
        }
        return indices;
    }
    
    /**
     * Sets the module index / sets the ownership of this
     * data crow object  
     */
    protected abstract void setModuleIndex();
    
    /**
     * Initialize all user settings on this data crow object 
     */
    protected abstract void initializeSettings();
    
    protected abstract void setSelectTableDef();
    
    /**
     * Sets the database table name for this object 
     */
    protected abstract void setTableName();    

    /**
     * Apply the IDs on this object and its children
     */
    public void setIDs() {
        String sID = Utilities.getUniqueID().substring(0, 15);
        setValue(DCObject._ID, sID);
        for (Iterator iter = alChildren.iterator(); iter.hasNext(); ) {
            DCObject child = (DCObject) iter.next();
            child.setIDs();
            child.setValue(child.getParentReferenceFieldIndex(), sID);
        }
    }    
    
    /**
     * Retrieves the value for the given field index
     * @param index field index
     */
    private DCValue getValueDef(int index) {
        return (DCValue) alValues.get(index);
    }
    
    /**
     * Creates the value objects (empty value objects)
     */
    private void initializeValues() {
        for (int i = 0; i < getModule().getFieldCount(); i++) {
            alValues.add(new DCValue());
        }
    }    
    
    /**
     * Check if all required fields have been filled
     */
    private void validateRequiredFields() throws ValidationException {
        String s = this.getUnfilledRequiredFields();

        if (s.trim().length() > 1) {
            String message;
            if (getRow() != -1) {
            	//shown by the table panels (ui row ID is set)
                message = Resources.getText(DCMessages.msgIncompleteRecRow,
                		  new String[] {s, toString(), String.valueOf(getRow() + 1)});
            } else {
                //shown by item forms and the likes
                message = Resources.getText(DCMessages.msgIncompleteRec, s);
            }
            throw new ValidationException(message);
        }
    }
}
