/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.cdimport;

import java.io.File;
import java.util.Enumeration;
import java.util.Vector;

import net.sf.dc.console.da.ImportCDDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.modules.DCModules;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.messages.Messages;
import net.sf.dc.messages.MessagesWriter;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.util.DirectoryReader;
import net.sf.ext.util.Resources;

/**
 * Imports data from a CD (or harddisk) location into the insert table
 * of the owner module
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.00
 */
public abstract class ImportCDProcess extends Thread {

    protected static final WorkFlow wf = new WorkFlow();

    private final ImportCDDialog ui;
    
    protected String sourceDir = "";
    protected boolean confirmEachDir = false;
    protected String cdName = "";
    
    private boolean keepOnRunning = true;

    /**
     * Initializes (but does  not run) the CD import process
     * @param ui used for displaying the progress
     * @param sourceDir directory which is used as source for the
     *        import
     * @param confirmEachDir indicates whether for each directory
     *        confirmation should be asked if the underlying data
     *        of this directory should be read or not.
     * @param nameOfCD cd name (used as the location name)
     */    
    public ImportCDProcess( ImportCDDialog ui, 
                            String sourceDir,
                            boolean confirmEachDir,
                            String cdName) {
    	this.ui = ui;
        
        if (!sourceDir.endsWith(File.separator)) {
            this.sourceDir = sourceDir;
        } else {
            this.sourceDir = sourceDir.substring(0, sourceDir.length() -1);
        }
        this.confirmEachDir = confirmEachDir;
        this.cdName = cdName;
    }
    
    /**
     * Starts the CD Import process
     */
    public void run() {
        try {
            ui.addMessage(Resources.getText(DCMessages.msgImportStarts));
            ui.addMessage(Resources.getText(DCMessages.msgImportReadingDirs));
            Vector filesAndDirs = DirectoryReader.read( sourceDir, 
                                                        confirmEachDir,
                                                        true,
                                                        true,
                                                        getExtensions(),
                                                        false);
            writeOutput(filesAndDirs);
        } catch (Exception exp) {
            exp.printStackTrace();
            ui.addMessage(DCMessages.msgProcessCrash);
            MessagesWriter._useLogFile = true;
            Messages.setMessage(exp);
            new MessageBox(Resources.getText(DCMessages.msgProcessCrash), 
                           MessageBox._ERROR);
        }
    }   
    
    public abstract String[] getExtensions();
    
    private void finish() {
        ui.addMessage(Resources.getText(DCMessages.msgImportCompleted));
    }
    
    public void cancel() {
    	keepOnRunning = false;
        finish();
    }
    
    /**
     * Parses the source files for information and adds the parsed
     * information to the insert table of the owner module.
     * @param filenames files to be parsed
     */
    public void writeOutput(Vector filenames) {
        try {
            ui.addMessage(Resources.getText(DCMessages.msgImportFoundXResults,
                          "" + filenames.size()));
            ui.initProgressBar(filenames.size());
            int counter = 1;
            for (   Enumeration enum = filenames.elements(); 
                    enum.hasMoreElements() && keepOnRunning; 
                    counter++) {
                
                String sFilename = (String) enum.nextElement();
                ui.addMessage(Resources.getText(DCMessages.msgProcessingFileX, sFilename));
                DCObject dco = parseInformation(sFilename);
                
                if (dco != null) {
                    DCModule module = DCModules.getModule(getModule());
                    module.getInsertPanel().addObject(dco);
                }
                try {
                	sleep(100);
                } catch (Exception ignore) {}
                
                ui.updateProgressBar(counter);
            }
        } catch (Exception exp) {
            new MessageBox(ui, Resources.getText(DCMessages.msgProcessCrash), 
                           MessageBox._ERROR);
            Messages.setMessage(exp);
        } finally {
        	filenames.clear();
        	filenames = null;
            ui.addMessage(Resources.getText(DCMessages.msgImportStops));
            ui.allowActions();
        }
    }    
    
    /**
     * Parses the file for its information. Needs to be implemented
     * by the extending classes 
     * @param sFilename file to be parsed
     */
    protected abstract DCObject parseInformation(String sFilename);
    
    /**
     * The UI class which will be updated with the results of the import 
     */
    protected abstract int getModule();
}
