/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.util.Vector;

import net.sf.dc.console.audiocd.ImportAudioCDDialog;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.Repository;
import net.sf.dc.core.modules.DCModule;
import net.sf.dc.core.objects.audiocd.AudioCD;
import net.sf.dc.freedb.Freedb;
import net.sf.dc.http.covers.AmazonCoverSource;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.dc.wf.WorkFlow;
import net.sf.ext.jflexsett.settings.Settings;
import net.sf.ext.util.DiscIDReader;
import net.sf.ext.util.Resources;

/**
 * Searches the FreeDB server by using keywords or the disc ID of an Audio CD
 * @author Robert Jan van der Waals
 */
public class FreeDBAudioCDUpdate extends InternetUpdateProcess {
    
    private final DCModule module;
    private final WorkFlow wf = new WorkFlow();
    private final boolean insert;
    
    private final ImportAudioCDDialog ui;
    
    /**
     * Initializes this dialog 
     * @param type searchtype
     * @param ui the ui to whoch messages will be send
     * @param module the module from which this dialog is started
     */
    public FreeDBAudioCDUpdate(ImportAudioCDDialog ui, DCModule module) {
        this.ui = ui;
        this.module = module;
        this.insert = true;
    }
    
    /**
     * Get the server name from the settings
     * @return qualified server name
     */
    private String getServer() {
        return Settings.getValueAsString(Repository.settings.stFreeDBServer);   
    }
    
    /**
     * Returns an Freedb instance. Can be used to start operations not present in this
     * class
     */
    public Freedb getFreedbInstance() {
        Freedb freeDb = new Freedb(getServer(), ui);
        return freeDb;
    }
    
    /**
     * Returns all the categories as known by the FreeDB server
     * @return array of category names
     */
    public String[] getCategories() {
        try {
        	Freedb freeDb = new Freedb(getServer(), ui);
        	return freeDb.getCategories();
        } catch (Exception exp) {
        	Messages.setMessage(exp);
        }
        return new String[0];
    }
    
    /**
     * Starts the search thread. Either the keywords or the disc id is/are used
     */
    public void run() {
		discIdSearch();
    }
    
    /**
     * Search the FreeDB server with the Disc ID. Messages and errors are send to the
     * UI. Progress in general is also displayed in the UI
     */
    private void discIdSearch() {
        DiscIDReader discIdReader = new DiscIDReader();
        String discId = "";
        try {
        	discId = discIdReader.readDiscID();
            ui.addMessage(Resources.getText(DCMessages.msgFoundDiscID, discId));
        } catch (Exception exp) {
            new MessageBox(Resources.getText(DCMessages.msgErrorDiscID, exp.getMessage()), 
                           MessageBox._ERROR);
            Messages.setMessage(exp);
            ui.addMessage(Resources.getText(DCMessages.msgQueryAborted));
            return;
        }
        
        if (!discIdReader.isSuccess()) {
            new MessageBox(Resources.getText(DCMessages.msgCouldNotReadDiscID), MessageBox._WARNING);
            ui.addMessage(Resources.getText(DCMessages.msgQueryAborted));
            return;
        }
        
        String server = getServer();
        ui.addMessage(Resources.getText(DCMessages.msgConnectingToServer, server));
        Freedb freeDb = new Freedb(server, ui);
        
        int index = discId.indexOf(" ");
        discId = discId.replaceAll(" ", "+");

        try {
            AudioCD[] audioCDs = freeDb.queryDiscId(discId);
            audioCDs = discardEquals(audioCDs);
            
            ui.addMessage(Resources.getText(DCMessages.msgFoundXResults, 
                                            String.valueOf(audioCDs.length)));
            
            if (audioCDs.length == 0) {
                new MessageBox(Resources.getText(DCMessages.msgNoResultsForDiscID, discId), 
                               MessageBox._WARNING);
            } else {
                for (int i = 0; i < audioCDs.length; i++) {
                    AudioCD audioCD = audioCDs[i];
                	AmazonCoverSource amazon = new AmazonCoverSource(ui, audioCD);
                	amazon.search();
                	ui.addObject(audioCD);
                    ui.addMessage(Resources.getText(DCMessages.msgParsingSuccessfull, 
                                  audioCD.getValue(AudioCD._A_TITLE).toString()));                    
                }
            }            
        } catch (Exception exp) {
            new MessageBox(Resources.getText(DCMessages.msgFreedbError, exp.getMessage()), 
                           MessageBox._ERROR);
            Messages.setMessage(exp);
            return;
        }
    }
    
    private AudioCD[] discardEquals(AudioCD[] audioCDs) {
        Vector v = new Vector();
    	for (int i = 0; i < audioCDs.length; i++) {
            AudioCD audioCD = audioCDs[i];
            boolean unique = true;
            for (int j = 0; j < audioCDs.length && unique; j++) {
                if (i != j && audioCD.equals(audioCDs[j])) {
                	unique = false;
                }
            }
            if (unique) {
            	v.addElement(audioCD);   
            }
        }
        
        if (v.size() == 0 && audioCDs.length > 0) {
        	v.addElement(audioCDs[0]);
        }
        
        audioCDs = null;
        AudioCD[] uniqueCDs = new AudioCD[v.size()];
        uniqueCDs = (AudioCD[]) v.toArray(uniqueCDs);
        return uniqueCDs;
    }
    
    /**
     * Search the FreeDB server with the keywords. Messages and errors are send to the
     * UI. Progress in general is also displayed in the UI
     */
    /*private void keyWordSearch(String keyWords) {
        String server = getServer();
        ui.addMessage(Resources.getText(DCMessages.msgConnectingToServer, server));
        Freedb freeDb = new Freedb(server, ui);        
        
        try {
            FreedbQueryResult[] results = freeDb.queryKeywords(keyWords);
            ui.addMessage(Resources.getText(DCMessages.msgFoundXResults, 
                          String.valueOf(results.length)));
            
            if (results == null || results.length == 0) {
                new MessageBox(Resources.getText(DCMessages.msgNoResultsForKeywords, keyWords), 
                               MessageBox._WARNING);
            } else {
                ui.addMessage(Resources.getText(DCMessages.msgTransferringXResults, 
                              String.valueOf(results.length)));
                ui.setSearchResult(results);
            }
        } catch (Exception exp) {
            new MessageBox(Resources.getText(DCMessages.msgFreedbError, exp.getMessage()), 
                           MessageBox._ERROR);
            Messages.setMessage(exp);
            return;
        }            
    }*/    
}
