/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.dc.processes.http;

import java.net.URL;
import java.util.Vector;

import net.sf.dc.console.InternetSearchForm;
import net.sf.dc.console.dialogs.MessageBox;
import net.sf.dc.core.objects.DCObject;
import net.sf.dc.core.objects.movie.Movie;
import net.sf.dc.http.HttpConnection;
import net.sf.dc.messages.Messages;
import net.sf.dc.resources.DCMessages;
import net.sf.ext.util.Resources;
import net.sf.ext.util.StringUtils;

/**
 * Access www.imdb.com to gather movie specific information
 *
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version 1.6
 */
public class IMDBMovieUpdate extends InternetUpdateProcess {

    private final InternetSearchForm ui;
    private String searchString;

    /**
     * initializes the IMDB query
     * @param searchString the string on which will be searched
     */
    public IMDBMovieUpdate(InternetSearchForm ui, String searchString, boolean broadband) {
        this.ui = ui;
        this.searchString = searchString.replace(' ', '+');
    } 
    
    public void run() {
    	String[] IDs = getIDs();
        ui.initProgressBar(IDs.length);
        
        if (IDs.length == 0) {
            new MessageBox(Resources.getText(DCMessages.msgNoResultsForKeywords,
                           searchString), MessageBox._WARNING);
            return;
        }
        
        ui.addMessage(Resources.getText(DCMessages.msgFoundXResults, 
                      String.valueOf(IDs.length)));      
        ui.addMessage(Resources.getText(DCMessages.msgStartParsingXResults, 
                      String.valueOf(IDs.length)));          
        
        for (int i = 0; i < IDs.length && !cancelled; i++) {
            try {
            	DCObject dco = parseInformation(IDs[i]);
                ui.addMessage(Resources.getText(DCMessages.msgParsingSuccessfull, 
                              dco.getValue(Movie._A_TITLE).toString()));          
                ui.updateProgressBar(i + 1);
            	ui.addObject(dco);
            } catch (Exception exp) {
            	ui.addMessage(Resources.getText(DCMessages.msgParsingError, exp.getMessage()));
                Messages.setMessage(exp);
                ui.updateProgressBar(i + 1);
            }
        }
    }

    /**
     * Searches for the maching ID's from the search result(s)
     */
    public String[] getIDs() {
        Vector vKeys = new Vector(0);
        
        try {
            searchString = searchString.replaceAll(" ", "%20");
            
            ui.addMessage(Resources.getText(DCMessages.msgConnectingToServer, 
                          "www.imdb.com"));
            
            URL url = new URL("http://www.imdb.com/Find?" + searchString);
    
            HttpConnection connection = new HttpConnection(url);
            String sWebPage = connection.getString(url);
            connection.close();
    
            String sTemp = sWebPage;
            if (sTemp.indexOf("/title/tt") > -1) {
                int temp = 0;
                while (temp != -1) {
                    temp = sTemp.indexOf("/title/tt");
                    if (temp != -1) {
                        sTemp = sTemp.substring(temp + 9, sTemp.length());
                        String key  = sTemp.substring(0, 7);
                        if (!vKeys.contains(key)) {
                        	vKeys.add(key);
                        }
                    }
                }
            }
        } catch (Exception e) {
            new MessageBox(Resources.getText(DCMessages.msgCouldNotConnectToImdb), MessageBox._ERROR);
            Messages.setMessage(e);
            ui.addMessage(Resources.getText(DCMessages.msgCouldNotConnectToImdb));
        }

        String[] keys = new String[vKeys.size()];
        return (String[]) vKeys.toArray(keys);
    }
    
    /**
     * Reads the information from www.imdb.com
     * @param sID the movie ID from www.imdb.com
     */
    public Movie parseInformation(String sID) throws Exception {
        Movie movie = new Movie();
        
        movie.setValue(Movie._C_WEBPAGE, "http://www.imdb.com/Title?" + sID);
        movie.setValue(Movie._B_DESCRIPTION, getDescription(sID));

        URL url = new URL("http://www.imdb.com/Title?" + sID);
        HttpConnection connection = new HttpConnection(url);
        String sWebPage = connection.getString(url);
        connection.close();
        
        String sTitle = StringUtils.getValueBetweenStrings("<title>", "</title>", sWebPage);
        movie.setValue(Movie._A_TITLE, StringUtils.convertHttpString(sTitle));
        movie.setValue(Movie._H_YEAR,  StringUtils.getValueBetweenStrings("(", ")", sTitle));
 
        String sDirected = 
            StringUtils.getValueBetweenStrings("Directed by", "</table>", sWebPage);
        sDirected = 
            StringUtils.getValueBetweenStrings("/" + '"' + ">", "</a><br>", sDirected);
        movie.setValue(Movie._F_DIRECTOR, StringUtils.convertHttpString(sDirected));

        String sGenre =
            StringUtils.getValueBetweenStrings("Genre", "<br>", sWebPage); 
        int iGenre = sGenre.indexOf("/\">");
        String s = "";
        while (iGenre != -1) {
            sGenre = sGenre.substring(iGenre + 3);
            s += sGenre.substring(0, sGenre.indexOf("<")) + (",");
            iGenre = sGenre.indexOf("/\">");
        }
        sGenre = s.length() > 1 ? s.substring(0, s.length() -1) : s;
        movie.setValue(Movie._D_GENRE, StringUtils.convertHttpString(sGenre));
            
            
        int iRuntime = sWebPage.indexOf("Runtime");
        String sRuntime = "";
        if (iRuntime != -1) {
            String sTemp = sWebPage.substring(iRuntime + 13);
            if (sTemp.indexOf("<") < sTemp.indexOf(" ")) {
                sRuntime = sTemp.substring(0, sTemp.indexOf("<"));
            } else {
                iRuntime = sTemp.indexOf("\t");
                if (iRuntime != -1) {
                    if (sTemp.indexOf(" ") < iRuntime) {
                        sRuntime = sTemp.substring(0, sTemp.indexOf(" "));
                    } else {
                        sRuntime = sTemp.substring(0, sTemp.indexOf("\t"));
                    }
                } else {
                     sRuntime = sTemp.substring(0, sTemp.indexOf(" "));
                }
            }
            try {
                iRuntime = new Integer(sRuntime).intValue() * 60;
                sRuntime = "" + iRuntime;
            } catch (NumberFormatException e) {
                sRuntime = "";
            }
        }
        
        movie.setValue(Movie._I_PLAYLENGTH, sRuntime);
        
        String sCountry = StringUtils.getValueBetweenStrings("Country:</b>", "/a>", sWebPage);
        sCountry = StringUtils.getValueBetweenStrings("/" + '"' + '>', "<", sCountry);
        movie.setValue(Movie._G_COUNTRY, StringUtils.convertHttpString(sCountry)); 
            

        int iActors = sWebPage.indexOf("ast overview");
        if (iActors == -1)
            iActors = sWebPage.indexOf("Complete credited cast:");
        if (iActors != -1) {
            String sTemp = sWebPage.substring(iActors);
            iActors = sTemp.indexOf("href=\"");
            sTemp = sTemp.substring(iActors, sTemp.indexOf("</table>"));
            iActors = sTemp.indexOf("(more)");
            if (iActors != -1) {
                sTemp = sTemp.substring(0, iActors - 50);
            }

            iActors = sTemp.indexOf("\">");
            String actors = "";
            while (iActors != -1) {
                try {
                    sTemp = sTemp.substring(iActors + 2);
                    actors += sTemp.substring(0, sTemp.indexOf("<")) + (", ");
                    
                    iActors = sTemp.indexOf("\"top\">");
                    if (iActors > -1) {
                        sTemp = sTemp.substring(iActors + 6);
                    }
                    
                    iActors = sTemp.indexOf("<");
                    if (iActors > 2) {
                        actors += sTemp.substring(0, sTemp.indexOf("<") - 1) + (", ");
                    }
                    
                    iActors = sTemp.indexOf("href");
                    if (iActors > -1) {
                        sTemp = sTemp.substring(iActors + 4);
                    }
                } catch (Exception exp) {
                    Messages.setMessage(exp);
                }
                
                iActors = sTemp.indexOf("\">");
            }
            movie.setValue(Movie._U_PICTUREFRONT, getPoster(sID));
            movie.setValue(Movie._E_ACTORS, 
                           StringUtils.convertHttpString((actors.substring(0, actors.length() - 1))));
        }
        
        return movie;
    }

    /**
     * Gets the description for this movie
     * @param sID imdb movie ID
     */
    public String getDescription(String sID) {
        try {
            URL url = new URL("http://www.imdb.com/Plot?" + sID);
            HttpConnection connection = new HttpConnection(url);
            String sPlot = connection.getString(url);
            connection.close();
            
            String s = "<p class=" + '"' + "plotpar" + '"' + ">";
            int index = sPlot.indexOf(s);
            if (index > -1) {
                sPlot = StringUtils.getValueBetweenStrings(s, "</p>", sPlot);
                return StringUtils.convertHttpString(sPlot);
            } 
        } catch (Exception ignore) {}
        return "";
    }    
    
    /**
     * Retrieves the movie poster from imdb.com
     * @param sID the imdb movie ID
     */
    private String getPoster(String sID) {
        try {
            URL url = new URL("http://www.imdb.com/title/tt" + sID + "/posters");
            HttpConnection connection = new HttpConnection(url);
            String picture = connection.getString(url);
            connection.close();
            
            int iPicture = picture.indexOf("Poster images shown are from");
            if (iPicture > -1) {
                picture = picture.substring(iPicture);
                picture = StringUtils.getValueBetweenStrings("background=" + '"', '"' +">", picture);
                
                String picUrl = "posters.imdb.com";
                int iPicture2 = picture.indexOf(picUrl);
                if (iPicture2 != -1) {
                    String picUrl2 = picture.substring(iPicture2 + picUrl.length());
                    url = new URL("http://posters.imdb.com" + picUrl2);
                    connection = new HttpConnection(url);
                    String image = connection.getImageAsBase64();
                    
        	    	if (image.length() < 1200) {
        	    		return "";
        	    	} else {
        	    		return image;	
        	    	}
                } 
            }
        } catch (Exception exp) {
            ui.addMessage(Resources.getText(DCMessages.msgPictureCouldNotBeRetrieved));
            Messages.setMessage(exp);
        }
        
        return "";
    }
}
