/******************************************************************************
 *                                     __                                     *
 *                              <-----/@@\----->                              *
 *                             <-< <  \\//  > >->                             *
 *                               <-<-\ __ /->->                               *
 *                               Data /  \ Crow                               *
 *                                   ^    ^                                   *
 *                       (c) 2003 The Data Crow team                          *
 *                              info@datacrow.net                             *
 *                                                                            *
 *                                                                            *
 *       This library is free software; you can redistribute it and/or        *
 *        modify it under the terms of the GNU Lesser General Public          *
 *       License as published by the Free Software Foundation; either         *
 *     version 2.1 of the License, or (at your option) any later version.     *
 *                                                                            *
 *      This library is distributed in the hope that it will be useful,       *
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
 *           Lesser General Public License for more details.                  *
 *                                                                            *
 *     You should have received a copy of the GNU Lesser General Public       *
 *    License along with this library; if not, write to the Free Software     *
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA   *
 *                                                                            *
 ******************************************************************************/

package net.sf.ext.jflexsett.settings;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.util.Vector;

import javax.swing.JComponent;

import net.sf.dc.console.components.ComponentFactory;
import net.sf.dc.core.Repository;
import net.sf.dc.messages.Messages;

/**
 * A Settings definition. The setting is always added to a file
 * (the settings file) and if specified, the setting is also
 * editable thru the SettingsView.
 * 
 * @author Robert-Jan van der Waals
 * @since 1.3
 * @version DC0.50
 */
public class Setting {

    private int iDataType;
    private String sKey;
    private Object oValue;

    private int settingsGroup = 0; 
    private int parentGroup = -1; // no parent as default    
    private int componentType = ComponentFactory._SHORTTEXTFIELD;
    private String helpText = "";
    private boolean displayLabel = true; 
    private boolean showToUser = true;
    private String labelText;
    
    private JComponent component = null;

    /**
     * Creates a setting
     * 
     * @param iDataType the data type of the value
     * @param sKey a unique identifier, also used for the settings file
     * @param oValue the default value for thsi setting
     * @param component the UI component
     * @param helptext helptext for this setting, used as tooltip
     * @param labeltext the display text for the label
     * @param displaylabel show the label on screen ?
     * @param showToUser is the setting meant to be display in the panel ?
     */
    public Setting(int iDataType,
            String sKey,
            Object oValue,
            JComponent component,
            String helpText,
            String labelText,
            boolean displayLabel,
            boolean showToUser) {
     
        this(iDataType, 
             sKey, 
             oValue, 
             -1, 
             helpText, 
             labelText,
             displayLabel,
             showToUser);
        this.component = component;
    }
    
    
    /**
     * Creates a setting
     * 
     * @param iDataType the data type of the value
     * @param sKey a unique identifier, also used for the settings file
     * @param oValue the default value for thsi setting
     * @param componentType the UI component
     * @param helptext helptext for this setting, used as tooltip
     * @param labeltext the display text for the label
     * @param displaylabel show the label on screen ?
     * @param showToUser is the setting meant to be display in the panel ?
     */
    public Setting(int iDataType,
                   String sKey,
                   Object oValue,
                   int componentType,
                   String helpText,
                   String labelText,
                   boolean displayLabel,
                   boolean showToUser) {

        this.iDataType = iDataType;
        this.sKey = sKey;
        this.oValue = oValue;
        this.componentType = componentType;
        this.displayLabel = displayLabel;
        this.showToUser = showToUser;
        this.labelText = labelText;
        this.helpText = helpText;
    }
    
    public String getLabelText() {
        return labelText;   
    }
    
    public boolean displayLabel() {
        return displayLabel;   
    }
    
    public boolean showToUser() {
        return showToUser;   
    }
    
    public int getSettingsGroup() {
        return settingsGroup;   
    }

    public int getSettingsGroupParent() {
        return parentGroup;   
    }
    
    public JComponent getUIComponent() {
        if (component == null) {
            component = ComponentFactory.getComponent(componentType, labelText, 400);
        }
        return component;
    }

    public int getDataType() {
        return iDataType;
    }

    public String getKey() {
        return sKey;
    }

    public Object getValue() {
        return oValue;
    }
    
    public String getHelpText() {
        return helpText;   
    }

    public String getValueAsString() {
        String toString = "";
        switch (iDataType) {
            case Repository.datatypes._DIMENSION:
                int x = new Double(((Dimension) oValue).getWidth()).intValue();
                int y = new Double(((Dimension) oValue).getHeight()).intValue();

                toString = "" + x + '/' + y;
                break;
            case Repository.datatypes._FONT:
                boolean bold = ((Font) oValue).isBold();
                toString =  toString +
                            ((Font) oValue).getName() + '/' +
                            ((Font) oValue).getSize() + '/' +
                            ((Font) oValue).isBold();
                break;
            case Repository.datatypes._COLOR:
                toString =  toString +
                            ((Color) oValue).getRed() + '/' +
                            ((Color) oValue).getGreen() + '/' +
                            ((Color) oValue).getBlue();
                break;
            case Repository.datatypes._INTEGER:
                toString =  toString +
                            String.valueOf(((Integer) oValue).intValue());
                break;
            case Repository.datatypes._STRINGARRAY:
                String[] sArray = (String[]) oValue;
                for (int i = 0; i < sArray.length; i++) {
                    toString = toString + sArray[i] + '#';
                }
                break;
            case Repository.datatypes._INTEGERARRAY:
                int[] iArray = (int[]) oValue;
                for (int i = 0; i < iArray.length; i++) {
                    toString = toString + iArray[i] + '#';
                }
                break;                
            default:
                toString =  toString + oValue.toString();
        }
        return toString;
    }

    public void setValue(Object oValue) {
        this.oValue = oValue;
    }

    public void setStringAsValue(String sValue) {
        try {
            switch (iDataType) {
                case Repository.datatypes._DIMENSION:
                    int index1 = sValue.indexOf("/");
                    String testX = sValue.substring(0, index1);
                    String testY = sValue.substring(index1 + 1, sValue.length());

                    int x = Integer.valueOf(testX).intValue();
                    int y = Integer.valueOf(testY).intValue();

                    oValue = new Dimension(x, y);
                    break;
                case Repository.datatypes._COLOR:
                    int index = sValue.indexOf("/");
                    String testRed = sValue.substring(0, index);
                    String testTemp = sValue.substring(index + 1, sValue.length());
                    index = testTemp.indexOf("/");
                    String testGreen = testTemp.substring(0, index);
                    String testBlue = testTemp.substring(index + 1, testTemp.length());

                    int red = Integer.valueOf(testRed).intValue();
                    int green = Integer.valueOf(testGreen).intValue();
                    int blue = Integer.valueOf(testBlue).intValue();

                    oValue = new Color(red, green, blue);
                    break;
                case Repository.datatypes._FONT:
                    int index2 = sValue.indexOf("/");
                    String fontName = sValue.substring(0, index2);
                    
                    String tempie = sValue.substring(index2 + 1, sValue.length());
                    index2 = tempie.indexOf("/");
                    String testSize = tempie.substring(0, index2);
                    String testBold = tempie.substring(index2 + 1, tempie.length());                    

                    int size = Integer.valueOf(testSize).intValue();
                    boolean bold = testBold.toLowerCase().equals("true") ? true : false;
                    if (bold) {
                    	oValue = new Font(fontName, Font.BOLD, size);
                    } else {
                        oValue = new Font(fontName, Font.PLAIN, size);
                    }
                    break;
                case Repository.datatypes._INTEGER:
                    oValue = new Integer(Integer.parseInt(sValue));
                    break;
                case Repository.datatypes._BOOLEAN:
                    oValue = Boolean.valueOf(sValue);
                    break;
                case Repository.datatypes._INTEGERARRAY:
                    Vector vInt = new Vector(0);

                    index = sValue.indexOf("#");
                    while (index > -1) {
                        vInt.add(new Integer(sValue.substring(0, index)));
                        sValue = sValue.substring(index + 1, sValue.length());
                        index = sValue.indexOf("#");
                    }
                    Integer[] iFields = new Integer[vInt.size()];
                    iFields = (Integer[]) vInt.toArray(iFields);
                    
                    int[] intArray = new int[iFields.length];
                    for (int i = 0; i < iFields.length; i++) {
                        intArray[i] = iFields[i].intValue();
                    }
                    oValue = intArray;
                    break;                    
                case Repository.datatypes._STRINGARRAY:
                    Vector fields = new Vector(0);

                    index = sValue.indexOf("#");
                    while (index > -1) {
                        fields.add(sValue.substring(0, index));
                        sValue = sValue.substring(index + 1, sValue.length());
                        index = sValue.indexOf("#");
                    }

                    String[] sFields = new String[fields.size()];
                    oValue = fields.toArray(sFields);
                    break;
                default:
                    oValue = sValue;
            }
        } catch (Exception exp) {
            Messages.setMessage(exp);
        }
    }
}
