package net.sf.ext.mediamanager;

import java.io.DataInputStream;

import net.sf.ext.util.Resources;

public class FilePropertiesRIFF extends FileProperties {

    private final int RIFF_AVI = 0x20495641;
    private final int AVI_LIST = 0x5453494c;
    private final int AVI_movi = 0x69766f6d;
    private final int AVI_avih = 0x68697661;
    private final int AVI_avih_SIZE = 0x00000038;
    private final int DX50_avih_SIZE = 0x00000040;
    private final int AVI_strh = 0x68727473;
    private final int AVI_strh_SIZE = 0x00000038;
    private final int DX50_strh_SIZE = 0x00000040;
    private final int AVI_vids = 0x73646976;
    private final int AVI_auds = 0x73647561;
    private final int AVI_strf = 0x66727473;

    private int lastSubChunk = 0;

    private boolean quit = false;

    /**
     * Processes a file from the given DataInputStream.
     */
    public void process(DataInputStream dataStream) throws Exception {
        int streamSize = readUnsignedInt32(dataStream);
        int streamType = readUnsignedInt32(dataStream);
        if (streamType == RIFF_AVI) {
            processAviChunks(dataStream, streamSize);
        } 
    }

    /**
     * Processes n bytes of the AVI chunk.
     */
    private void processAviChunks(DataInputStream dataStream, int n)
        throws Exception {
        while (n > 0 && !quit) {
            int chunkType = readUnsignedInt32(dataStream);
            n -= 4;
            int chunkSize = readUnsignedInt32(dataStream);
            n -= 4;
            n -= chunkSize;
            switch (chunkType) {
                case (AVI_LIST) :
                    if (readUnsignedInt32(dataStream) == AVI_movi) {
                        quit = true;
                    } else {
                        processAviChunks(dataStream, chunkSize - 4);
                    }
                    break;
                case (AVI_avih) :
                    if (chunkSize != AVI_avih_SIZE && chunkSize != DX50_avih_SIZE) {
                        throw new Exception("RIFF file corrupted (avih chunk size is "
                                            + chunkSize
                                            + " and not 0x38|0x40 as expected).");
                    }
                    processAviAvih(dataStream, chunkSize);
                    break;
                case (AVI_strh) :
                    if (chunkSize != AVI_strh_SIZE && chunkSize != DX50_strh_SIZE) {
                        throw new Exception("RIFF file corrupted (strh chunk size is "
                                            + chunkSize + " and not 0x38|0x40 as expected).");
                    }
                    /* Get the sub chunk type... */
                    int subChunk = readUnsignedInt32(dataStream);
                    /* Processes according to the type... */
                    switch (subChunk) {
                        case (AVI_vids) :
                            processAviVids(dataStream, chunkSize - 4);
                            break;
                        case (AVI_auds) :
                            skipBytes(dataStream, chunkSize - 4);
                            lastSubChunk = AVI_auds;
                            break;
                        default :
                            skipBytes(dataStream, chunkSize - 4);
                            break;
                    }
                    break;
                case (AVI_strf) :
                    switch (lastSubChunk) {
                        case (AVI_auds) :
                            processAviSound(dataStream, chunkSize);
                            break;
                        default :
                            skipBytes(dataStream, chunkSize);
                            break;
                    }
                    lastSubChunk = 0;
                    break;
                default :
                    int m = chunkSize % 2;
                    if (m != 0) {
                        chunkSize += (2 - m);
                    }
                    skipBytes(dataStream, chunkSize);
                    break;
            }
        }
    }

    /**
     * Processes the AVI avih chunk.
     **/
    private void processAviAvih(DataInputStream dataStream, int chunkSize)
        throws Exception {
        int dwMicroSecPerFrame = readUnsignedInt32(dataStream);
        int dwMaxBytesPerSec = readUnsignedInt32(dataStream);
        skipBytes(dataStream, 8);
        int dwTotalFrames = readUnsignedInt32(dataStream);
        skipBytes(dataStream, 12);
        int dwWidth = readUnsignedInt32(dataStream);
        int dwHeight = readUnsignedInt32(dataStream);
        skipBytes(dataStream, chunkSize - 40);
        setVideoBitRate(Math.round((dwMaxBytesPerSec) * 8F / 1000F));

        if (dwMicroSecPerFrame > 0) {
            setDuration(Math.round((dwTotalFrames / 1000F)
                                 * (dwMicroSecPerFrame / 1000F)));
        }

        setVideoResolution(dwWidth + "x" + dwHeight);
        setVideoWidth(dwWidth);
        setVideoHeight(dwHeight);        
    }

    /**
     * Processes of avi_vids sub chunk.
     **/
    private void processAviVids(DataInputStream dataStream, int chunkSize)
        throws Exception {
        int fccHandler = readUnsignedInt32(dataStream);
        skipBytes(dataStream, 12);
        int dwScale = readUnsignedInt32(dataStream);
        int dwRate = readUnsignedInt32(dataStream);
        skipBytes(dataStream, chunkSize - 24);
        setVideoCodec(getVideoCodecName(fccHandler));

        setVideoFps((dwRate / dwScale));
    }

    /**
     * Processes of avi_strf on the same sub chuck that avi_auds.
     **/
    private void processAviSound(DataInputStream dataStream, int chunkSize) throws Exception {
        int wFormatTag = readUnsignedInt16(dataStream);
        int nChannels = readUnsignedInt16(dataStream);
        int nSamplesPerSec = readUnsignedInt32(dataStream);
        int nAvgBytesPerSec = readUnsignedInt32(dataStream);
        skipBytes(dataStream, chunkSize - 12);
        setAudioCodec(getAudioCodecName(wFormatTag));
        setAudioChannels(nChannels);
        setAudioRate(nSamplesPerSec);
        setAudioBitRate(Math.round(nAvgBytesPerSec * 8F / 1000F));
    }

    /**
     * Gets the video codec name from file based on the id.
     **/
    private String getVideoCodecName(int id) throws Exception {
        /* Transforms the id in a string... */
        StringBuffer buffer = new StringBuffer(4);
        for (int i = 0; i < 4; i++) {
            int c = id & 0xff;
            buffer.append((char) c);
            id >>= 8;
        }
        return new String(buffer);
    }

    /**
     * Gets the audio codec name from file based on the id.
     **/
    private String getAudioCodecName(int id) throws Exception {
        /* Transforms the id in a string... */
        StringBuffer buffer = new StringBuffer("RIF.0x");
        String value = Integer.toHexString(id);
        
        int i = 4 - value.length();
        while (i-- > 0) {
            buffer.append('0');
        }
        
        buffer.append(value);
        
        return Resources.getText(new String(buffer));
    }
}
