/*
 * Entagged Audio Tag library
 * Copyright (c) 2003-2005 Raphal Slinckx <raphael@slinckx.net>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package entagged.audioformats.mp3.util.id3frames;

import java.io.UnsupportedEncodingException;

import entagged.audioformats.generic.TagField;
import entagged.audioformats.mp3.Id3v2Tag;


public abstract class Id3Frame implements TagField {
	protected byte[] flags;
	protected byte version;
	
	public Id3Frame() {
		this.version = Id3v2Tag.ID3V23;
		createDefaultFlags();
	}
	
	public Id3Frame(byte[] raw, byte version) throws UnsupportedEncodingException {
	    byte[] rawNew;
		if(version == Id3v2Tag.ID3V23) {
			byte size = 2;
			
			if((raw[1]&0x80) == 0x80) {
				//Compression zlib, 4 bytes uncompressed size.
				size += 4;
			}
			
			if((raw[1]&0x80) == 0x40) {
				//Encryption method byte
				size += 1;
			}
			
			if((raw[1]&0x80) == 0x20) {
				//Group identity byte
				size += 1;
			}
			
			this.flags = new byte[size];
			for(int i = 0; i<size; i++)
				this.flags[i] = raw[i];
			rawNew = raw;
		} else {
			createDefaultFlags();
			rawNew = new byte[this.flags.length + raw.length];
			copy(this.flags, rawNew, 0);
			copy(raw, rawNew, this.flags.length);
		}
		
		this.version = version;
		
		populate(rawNew);
	}
	
	private void createDefaultFlags() {
		this.flags = new byte[2];
		this.flags[0] = 0;
		this.flags[1] = 0;
	}
	
	public byte[] getFlags() {
		return this.flags;
	}
	
	public byte[] getRawContent() throws UnsupportedEncodingException {
		return build();
	}
	
	public void isBinary(boolean b) { /* Never can choose if binary or not */ }
	
	public abstract String getId();
	public abstract boolean isBinary();
	protected abstract void populate(byte[] raw) throws UnsupportedEncodingException;
	protected abstract byte[] build() throws UnsupportedEncodingException;
	public abstract boolean isCommon();
	
	protected int indexOfFirstNull(byte[] b, int offset) {
		for(int i = offset; i<b.length; i++)
			if(b[i] == 0)
				return i;
		return -1;
	}
	
	protected void copy(byte[] src, byte[] dst, int dstOffset) {
		for(int i = 0; i<src.length; i++)
			dst[i+dstOffset] = src[i];
	}
	
	protected byte[] getIdBytes() {
		return getId().getBytes();
	}
	
	protected byte[] getSize(int size) {
		byte[] b = new byte[4];
		b[0] = (byte)( ( size >> 24 ) & 0xFF );
		b[1] = (byte)( ( size >> 16 ) & 0xFF );
		b[2] = (byte)( ( size >>  8 ) & 0xFF );
		b[3] = (byte)(   size         & 0xFF );
		return b;
	}
	
	protected String getString(byte[]b, int offset, int length, String encoding) throws UnsupportedEncodingException{
		return new String(b, offset, length, encoding);
	}
	
	protected byte[] getBytes(String s, String encoding) throws UnsupportedEncodingException{
		return s.getBytes(encoding);
	}
}
